#!/lib/klibc/bin/sh
# $Id: udev.get_unique_drive_id.sh 174 2004-06-16 19:11:53Z olh $
#set -x

# this script should provide the disk id 

# example for all:
# for i in `find /sys/block -name dev` ;do DEVPATH="`echo $i | sed -e 's@^/sys\|/dev@@g'`" $0 ; done

PATH=/lib/klibc/bin:/bin #/usr:/usr/sbin:/usr/bin:/sbin:/bin
SYSFS=/sys
export PATH
RESULT=1
TYPE=
OPWD="`pwd`"
# Check for 'pwd -P'
if $(pwd -P > /dev/null 2>&1); then
    pwd_cmd="pwd -P"
else
    pwd_cmd="pwd"
fi
full_sysfs_class_path=
full_sysfs_device_path=
is_partition=false
disk_id=

get_scsi_type () {
    local type

    read type < $1/type
    case "$type" in
	0)
	    # Direct-access device
	    devtype=disk
	    ;;
	1)
	    # Sequential access device
	    devtype=tape
	    ;;
	2)
	    # Printer device
	    devtype=printer
	    ;;
	3)
	    # Processor device
	    devtype=scanner
	    ;;
	4)
	    # Write-once device
	    devtype=cdrom
	    ;;
	5)
	    # CD-ROM device
	    devtype=cdrom
	    ;;
	6)
	    # Scanner device
	    devtype=scanner
	    ;;
	7)
            # Optical memory device
	    devtype=disk
	    ;;
	8)
	    # Medium changer devices
	    devtype=generic
	    ;;
	9)
	    # Communications device
	    devtype=generic
	    ;;
	10|11)
	    # Graphic arts pre-press device
	    devtype=generic
	    ;;
	12)
	    # Storage array controller
	    devtype=generic
	    ;;
	13)
	    # Enclosure service device
	    devtype=generic
	    ;;
	14)
	    # Simplified direct-access device
	    devtype=generic
	    ;;
	15)
	    # Optical card reader/writer device
	    devtype=generic;
	    ;;
	*)
	    devtype=generic;
	    ;;
    esac

    echo $devtype
}

if [ -z "$DEVPATH" -a -z "$1" ] ; then
exit 1
fi
if [ -z "$DEVPATH" ] ; then
	case "$1" in
		$SYSFS/*)
		DEVPATH="${1#$SYSFS}"
		;;
		*)
		DEVPATH=$1
		;;
	esac
fi

if [ ! -d $SYSFS$DEVPATH ] ; then
exit 1
fi
if [ ! -f $SYSFS$DEVPATH/dev ] ; then
exit 1
fi


case "$DEVPATH" in
	/block/*)
		TYPE=block
		;;
	/class/*)
		TYPE="${DEVPATH#/class/}"
		TYPE="${TYPE%%/*}"
		;;
	*)
		exit 1
		;;
esac
		

#
##
#


case "$TYPE" in
	scsi_generic)
	cd $SYSFS$DEVPATH/device/block || exit 1
	DEVPATH="`$pwd_cmd`"
	cd "$OPWD"
	DEVPATH="${DEVPATH#$SYSFS}"
	;;
esac

if [ -f $SYSFS$DEVPATH/range -a -L $SYSFS$DEVPATH/device ] ; then
	read major_minor < $SYSFS$DEVPATH/dev
	cd $SYSFS$DEVPATH/device
	fulldevpath="`$pwd_cmd`"
	if [ -d block ]; then
	    cd block
	    fullblockpath="`$pwd_cmd`"
	fi
	cd "$OPWD"
else
	if [ ! -L $SYSFS$DEVPATH/../device -a ! -f $SYSFS$DEVPATH/../range ] ; then
		exit 1
	fi
	read major_minor < $SYSFS$DEVPATH/../dev
	cd $SYSFS$DEVPATH/../device
	fulldevpath="`$pwd_cmd`"
	if [ -d block ]; then
	    cd block
	    fullblockpath="`$pwd_cmd`"
	fi
	cd "$OPWD"
	is_partition=true
fi

major=${major_minor%%:*}
minor=${major_minor##*:}
mynode=/dev/shm/diskid_tmpnode.$major.$minor.$$
devtype="generic"
#
#
handle_block () {
    case "$fulldevpath" in
	# USB devices are mapped onto SCSI
	*/usb[0-9]*/*)
            # Use the attributes from sysfs instead of doing a full-blown
            # INQUIRY as USB is notoriously slow and buggy
            if [ -f $fulldevpath/model ]; then
		read m < $fulldevpath/model
		model="`echo $m | sed -e 's@ @_@g;s@/@.@g'`"
	    fi
	    if [ -f $fulldevpath/vendor ]; then
		read v < $fulldevpath/vendor
		vendor="`echo $v | sed -e 's@ @_@g;s@/@.@g'`"
	    fi
	    if [ -f $fulldevpath/type ] ; then
		devtype="`get_scsi_type $fulldevpath`"
	    fi
	    if [ -z "$vendor" -o -z "$model" ]; then
		# No vendor or model attribute. 
		# Try to get the USB serial number
		d="`echo $fulldevpath |  sed -e 's@/[^/]\{1,\}/host[0-9]\{1,\}/[0-9]\{1,\}:[0-9]\{1,\}:[0-9]\{1,\}:[0-9]\{1,\}@@'`"
		serial=
		if [ -f $d/serial ] ; then
		    serial="`sed -e 's@^[ -]\{1,\}\|[ -]\{1,\}$@@g;s@[^abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_0123456789]@@g' < $d/serial`"
		    : serial XXX_${serial}_XXX
		    devtype=disk
		    disk_id="$serial"
		    serial="`echo $serial | sed -e 's@[ 0]\{1,\}@@g'`"
		fi
		if [ -z "$serial" ] ; then
		    # no serial, broken device
		    # has eventually binary junk in vpd
		    identifier=
		    if [ -f $d/product ] ; then
			product="`sed -e 's@^[ -]\{1,\}\|[ -]\{1,\}$@@g;s@[^abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_0123456789]@@g' < $d/product`"
		    fi
		    if [ -f $d/manufacturer ] ; then
			manufacturer="`sed -e 's@^[ -]\{1,\}\|[ -]\{1,\}$@@g;s@[^abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_0123456789]@@g' < $d/manufacturer`"
		    fi
		    if [ -z "$product" -o -z "$manufacturer" ] ; then
			read idvendor < $d/idVendor
			read idproduct < $d/idProduct
			identifier="0x${idvendor}-0x${idproduct}"
		    else
			identifier="${manufacturer}-${product}"
		    fi
		    devtype=disk
		    disk_id="${identifier}"
		fi
	    else
		disk_id="$vendor-$model"
	    fi
	    ;;
	# ide-scsi is CD only
	*/ide*/*/host*/*)
		: skip ide-scsi: $fulldevpath
		return 1
		;;
	*/iscsi/*)
	        # iSCSI disks
	        : looking at iSCSI $fulldevpath
	        p="`echo $fulldevpath |  sed -e 's@/iscsi/[0-9]\{1,\}:@/@'`"
		cil="`echo $p |  sed -e 's@^.*/@@'`"
		if [ -r /sys/class/scsi_host/iscsi/print_info ]; then
		    while read bus tgt lun ip port target; do
			if [ -n "$bus" -a "$bus" != "#" ]; then
			    if [ "${bus}:${tgt}:${lun}" = "$cil" ]; then
				disk_id="iscsi-$target-$lun"
			    fi
			fi
		    done < /sys/class/scsi_host/iscsi/print_info
		fi
		if [ "$disk_id" ]; then
		    RESULT=0
		    devtype=disk
		else
		    RESULT=1
		fi
	;;
	# scsi devices
	# firewire sbp2 has a GUID, but it may provide the serial of the disk
	*/host[0-9]*/[0-9]*:[0-9]*:[0-9]*:[0-9]*)
		# check for ieee1394 sbp2 
		# do not call scsi_id for firewire disks, it may hang
		if [ -f $fulldevpath/ieee1394_id ] ; then
			return 1
		fi
		: looking at scsi $fulldevpath
		disk_id="`scsi_id -gs $DEVPATH`"
		: disk_id XXX${disk_id}XXX
		case "$disk_id" in
			*Linux\ ATA-SCSI\ simulator)
			disk_id="`scsi_id  -p 0x80 -gs $DEVPATH`"
			: STATA disk_id XXX${disk_id}XXX
			;;
			*)
			;;
		esac
		# removes also leading and trailing whitespaces
		disk_id="`echo $disk_id | sed -e 's@ \+@_@g'`"
		if [ -f $fulldevpath/type ] ; then
		    devtype="`get_scsi_type $fulldevpath`"
		fi
	;;
	# ide devices
	*/ide[0-9]/[0-9].[0-9]*|*/ide[0-9][0-9]/[0-9][0-9].[0-9]*)
		
		: looking at ide $fulldevpath
		rm -f $mynode
		mknod -m 600 $mynode b $major $minor
		disk_id="`ata_identify $mynode`"
		rm -f $mynode
		if [ ! $? ]; then
		    disk_id=
		fi
		if [ -f /proc/ide/${fullblockpath##*/}/media ]; then
		    read media < /proc/ide/${fullblockpath##*/}/media
		    case "$media" in
			disk)
			    devtype=disk
			    ;;
			cdrom)
			    devtype=cdrom
			    ;;
			tape)
			    devtype=tape
			    ;;
			floppy)
			    devtype=disk
			    ;;
			*)
			    devtype=generic
			    ;;
		    esac
		fi
	;;
	*/css0/*)
	        # S/390 DASDs
		: looking at dasd $fulldevpath
		rm -f $mynode
		mknod -m 600 $mynode b $major $minor
	        disk_id="`/sbin/dasdview -j -f $mynode`"
		if [ ! $? ]; then
		    disk_id=
		else
		    disk_id=${disk_id%% *}
		    devtype=disk
		fi
		rm -f $mynode
	;;
	*)
		: $fulldevpath not handled
		return 1
	;;
esac
# done
return 0
}

case "$TYPE" in
	block)
	if ! handle_block; then
	    : not handled
	    exit 1
	fi

	if [ -z "$disk_id" -o "$devtype" = "generic" ]; then
	    : not handled
	    exit 1
	fi

	# Reset CDROM devices for backwards compability
	if [ "$devtype" = "cdrom" ]; then
	    devtype=disk
	fi

	# look for a partition number
	if [ ! -z "$disk_id" -a "$is_partition" = "true" ] ; then
		dp="`echo $DEVPATH | sed -e 's@^.*[^0-9]@@'`"
		case "$disk_id" in
			*[0-9])
			disk_id="${disk_id}p${dp}"
			;;
			*)
			disk_id="${disk_id}${dp}"
			;;
		esac
	fi
	if [ ! -z "$disk_id" ] ; then
		echo "$devtype/by-id/$disk_id"
	fi
	RESULT=0
	;;
	scsi_generic)
	if ! handle_block; then
	    : not handled
	    exit 1
	fi

	if [ -z "$disk_id" ]; then
	    : not handled
	    exit 1
	else
	    echo "generic/by-id/$disk_id"
	fi
	RESULT=0
	;;
	*)
	RESULT=1
	;;
esac
exit $RESULT
